
import xmlMap from "./maps/xml";
import legacyMap from "./maps/legacy";
import entityMap from "./maps/entities";
import decodeCodePoint from "./decode_codepoint.js";

function getStrictDecoder(map){
	var keys = Object.keys(map).join("|"),
	    replace = getReplacer(map);

	keys += "|#[xX][\\da-fA-F]+|#\\d+";

	var re = new RegExp("&(?:" + keys + ");", "g");

	return filteredReplacer(re, replace);
}

function getHTMLDecoder(map){
	var legacy = Object.keys(legacyMap)
		.sort(sorter);

	var keys = Object.keys(map)
		.sort(sorter);

	for(var i = 0, j = 0; i < keys.length; i++){
		if(legacy[j] === keys[i]){
			keys[i] += ";?";
			j++;
		} else {
			keys[i] += ";";
		}
	}

	var re = new RegExp("&(?:" + keys.join("|") + "|#[xX][\\da-fA-F]+;?|#\\d+;?)", "g"),
	    replace = getReplacer(entityMap);

	function replacer(str){
		if(str.substr(-1) !== ";") str += ";";
		return replace(str);
	}

	//TODO consider creating a merged map
	return filteredReplacer(re, replacer);
};

function filteredReplacer(re, replacer) {
	return function(str, fn){
		return String(str).replace(re, function(s, offset){
			var replaced = replacer(s);
			return (fn ? fn(s, replaced, offset) : replaced);
		});
	};
};

function sorter(a, b){
	return a < b ? 1 : -1;
}

var defaultReplacer = function(map, str){
	if(str.charAt(1) === "#"){
		if(str.charAt(2) === "X" || str.charAt(2) === "x"){
			return decodeCodePoint(parseInt(str.substr(3), 16));
		}
		return decodeCodePoint(parseInt(str.substr(2), 10));
	}
	return map[str.slice(1, -1)];
};

function getReplacer(map){
	return function(str){
		return defaultReplacer(map, str);
	};
}

var decodeHTML = getHTMLDecoder(entityMap);
var decodeXMLStrict = getStrictDecoder(xmlMap);
var decodeHTMLStrict = getStrictDecoder(entityMap);

export default {
	XML: decodeXMLStrict,
	HTML: decodeHTML,
	HTMLStrict: decodeHTMLStrict
};

