
import xmlMap from "./maps/xml";
import entitiesMap from "./maps/entities";

var inverseXML = getInverseObj(xmlMap),
    xmlReplacer = getInverseReplacer(inverseXML);

var inverseHTML = getInverseObj(entitiesMap),
    htmlReplacer = getInverseReplacer(inverseHTML);

function getInverseObj(obj){
	return Object.keys(obj).sort().reduce(function(inverse, name){
		inverse[obj[name]] = "&" + name + ";";
		return inverse;
	}, {});
}

function getInverseReplacer(inverse){
	var single = [],
	    multiple = [];

	Object.keys(inverse).forEach(function(k){
		if(k.length === 1){
			single.push("\\" + k);
		} else {
			multiple.push(k);
		}
	});

	//TODO add ranges
	multiple.unshift("[" + single.join("") + "]");

	return new RegExp(multiple.join("|"), "g");
}

var re_nonASCII = /[^\0-\x7F]/g,
    re_astralSymbols = /[\uD800-\uDBFF][\uDC00-\uDFFF]/g;

function singleCharReplacer(c){
	return "&#x" + c.charCodeAt(0).toString(16).toUpperCase() + ";";
}

function astralReplacer(c){
	// http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
	var high = c.charCodeAt(0);
	var low  = c.charCodeAt(1);
	var codePoint = (high - 0xD800) * 0x400 + low - 0xDC00 + 0x10000;
	return "&#x" + codePoint.toString(16).toUpperCase() + ";";
}

function filteredReplacer(re, replacer) {
	return function(str, fn){
		return String(str).replace(re, function(s, offset){
			var replaced = replacer(s);
			return (fn ? fn(s, replaced, offset) : replaced);
		});
	};
};

var re_xmlChars = getInverseReplacer(inverseXML);
var xmlFilter = filteredReplacer(re_xmlChars, singleCharReplacer);
var astralFilter = filteredReplacer(re_astralSymbols, astralReplacer);
var nonASCIIFilter = filteredReplacer(re_nonASCII, singleCharReplacer);

function getInverse(inverse, re){
	function inverseReplacer(name){
		return inverse[name];
	}

	return function(data, fn){
		data = filteredReplacer(re, inverseReplacer)(data, fn);
		data = astralFilter(data, fn);
		return nonASCIIFilter(data, fn);
	};
}

function escapeXML(data, fn){
	data = xmlFilter(data, fn);
	data = astralFilter(data, fn);
	return nonASCIIFilter(data, fn);
}

var XML = getInverse(inverseXML, xmlReplacer);
var HTML = getInverse(inverseHTML, htmlReplacer);

export default {
	escape: escapeXML,
	XML: XML,
	HTML: HTML
};

